from datetime import datetime
import time
import requests
import math
from config_manager import ConfigManager


_config = ConfigManager('config.conf')
_bitly_configs = _config.bitly_configs

class BitlyAPI:
    def __init__(self, token_id, logger):
        self.token0 = _bitly_configs[0].token
        self.token1 = _bitly_configs[1].token
        self.access_token = None

        if token_id == 0:
            self.access_token = self.token0
        else:
            self.access_token = self.token1

        self.base_url = 'https://api-ssl.bitly.com/v3/'

        self.logger = logger

    def get_end_url(self, bitly_url):

        params = {'access_token': self.access_token,
                  'shortUrl': bitly_url}

        endpoint = 'expand'

        r = self.get_response(endpoint, bitly_url, params)

        if r is not None:
            try:
                return r.json()['data']['expand'][0]['long_url']
            except Exception as exc:
                self.logger.log('bitly_api/get_clicks', 'Probably KeyError', str(exc))
                return 'Unknown'
        else:
            return 'Unknown'

    def get_created_at(self, bitly_url):

        params = {'access_token': self.access_token,
                  'shortUrl': bitly_url}

        endpoint = 'info'

        r = self.get_response(endpoint, bitly_url, params)

        if r is not None:
            try:
                date = datetime.fromtimestamp(r.json()['data']['info'][0]['created_at'])
                return date.strftime('%a %b %d %H:%M:%S +0000 %Y')
            except Exception as exc:
                self.logger.log('bitly_api/get_clicks', 'Probably KeyError', str(exc))
                return 'Unknown'
        else:
            return 'Unknown'

    def get_clicks_since_creation(self, bitly_url):
        return self.get_clicks(bitly_url, 'month', -1)

    def get_clicks_since_posting(self, bitly_url, posting_date):
        delta_time = (datetime.utcnow() - posting_date)
        units = math.ceil((delta_time.seconds / 3600) + (delta_time.days * 24))

        return self.get_clicks(bitly_url, 'hour', units)

    def get_clicks(self, bitly_url, unit, units):

        params = {'access_token': self.access_token,
                  'link': bitly_url,
                  'unit': unit,
                  'units': units}

        endpoint = 'link/referrers'

        r = self.get_response(endpoint, bitly_url, params)

        if r is not None:
            referrers = r.json()['data']['referrers']

            clicks_total = 0
            clicks_from_twitter = 0

            for ref in referrers:

                clicks_total += int(ref['clicks'])

                try:
                    if ref['referrer'].__contains__('//t.co/'):
                        clicks_from_twitter += int(ref['clicks'])
                except Exception as exc:
                    self.logger.log('bitly_api/get_clicks', 'Probably KeyError', str(exc))

            return [clicks_total, clicks_from_twitter]
        else:
            return [-1, -1]

    def get_response(self, endpoint_url, bitly_url, params):

        location = 'bitly_api/get_response, ' + endpoint_url

        url = self.base_url + endpoint_url

        request_status = None

        status_500_found = False
        request_error_found = False

        while True:
            try:
                r = requests.get(url=url, params=params)

                request_status = r.status_code

                status = r.json()['status_code']

                if status == 200:
                    return r

                elif status == 403:
                    self.logger.log(location, "Rate limit exceeded, will wait 5 min...", data=bitly_url)
                    time.sleep(60 * 5)

                elif status == 500:
                    if not status_500_found:
                        self.logger.log(location, "First server error, will retry in 5 sec", data=bitly_url)

                        time.sleep(5)
                        status_500_found = True
                    else:
                        self.logger.log(location, "Second server error, abandoning request", data=bitly_url)

                        return None
                else:
                    self.logger.log(location, "Exiting on status code " + str(status), data=bitly_url)

                    return None
            except:
                if not request_error_found:
                    self.logger.log(location,
                                    "First request error with code " + str(request_status) + ', will retry in 1 min',
                                    data=bitly_url)
                    time.sleep(60)
                    request_error_found = True
                else:
                    self.logger.log(location,
                                    "Second request error with code " + str(request_status) + ', abandoning request',
                                    data=bitly_url)
                    return None
