import sys
import os
from time import time, sleep
from datetime import datetime
from collection import TweetCollection, ExtraDataCollection
from threading import Thread
from logger import Logger
from collection_information import CollectionInformation
from extra_data_retriever import ExtraDataRetriever
from tweet_writer import TweetWriter
from twitter_api import TwitterApi
from tweet_queuer import TweetQueuer
from config_manager import ConfigManager

_config = ConfigManager('config.conf')
_collection_time = _config.collection_config.collection_time
_collection_interval = _config.collection_config.collection_interval
_delay_time = _config.collection_config.extra_data_delay

_collection_folder = _config.folders_config.collections_folder
_log_folder = _config.folders_config.logs_folder

def main():
    
    start_time = int(time())

    os.makedirs(_log_folder, exist_ok=True)
    os.makedirs(_collection_folder, exist_ok=True)

    logger = Logger(_log_folder + '/main_' + datetime.utcfromtimestamp(start_time).strftime('%Y-%m-%d_%H-%M') + '.log')
    
    [tweet_collections, extra_data_collections] = calc_collections(start_time)
    current_tweet_collection = tweet_collections.pop(0)
    next_extra_data_collection = extra_data_collections.pop(0)

    [twitter_api, tweet_queuer, stream_process, queuer_process, receive_connection, queue] = start_tweet_queing(start_time)

    tweet_writer = TweetWriter()
    tweet_writer.change_file(current_tweet_collection.file_path)
    collection_thread = Thread(target=collect_tweets, args=[queue, tweet_writer], daemon=True)
    collection_thread.start()

    retrieve_processes = []
    while True:
        if int(time()) >= current_tweet_collection.end_time:
            if (len(tweet_collections) > 0):
                current_tweet_collection = tweet_collections.pop(0)
                logger.log('Main/collect_tweets', 'Changing file!')
                tweet_writer.change_file(current_tweet_collection.file_path)

        if int(time()) >= next_extra_data_collection.start_time:
            extra_data_retriever = ExtraDataRetriever(_log_folder + '/extra_data_' + datetime.utcfromtimestamp(next_extra_data_collection.start_time).strftime('%Y-%m-%d_%H-%M') + '.log')
            retrieve_process = extra_data_retriever.start_retrieving(
                next_extra_data_collection.tweet_file_path,
                next_extra_data_collection.retweet_file_path,
                next_extra_data_collection.bitly_file_path,
                0
            )
            retrieve_processes.append(retrieve_process)

            if len(extra_data_collections) > 0:
                next_extra_data_collection = extra_data_collections.pop(0)
            else:
                break
        sleep(5)
        print('                              ', end="\r", flush=True)
        print('Queue length: ' + str(queue.qsize()), end="\r", flush=True)

    stream_process.terminate()
    stream_process.join()
    queuer_process.terminate()
    queuer_process.join()

    logger.log('Main/run', 'Will wait for extra data retrieve processes to end!')

    for p in retrieve_processes:
        p.join()

def collect_tweets(queue, writer):
    while True:
        if not queue.empty():
            writer.write(queue.get())


def calc_collections(start_time):
    number_of_collections = round(_collection_time / _collection_interval)
    tweet_collections = []
    extra_data_collections = []
    for i in range(number_of_collections):
        tweet_start_time = start_time + _collection_interval * 60 * i
        tweet_end_time = tweet_start_time + _collection_interval * 60
        tweet_file_path = _collection_folder + '/collection_' + datetime.utcfromtimestamp(tweet_start_time).strftime('%Y-%m-%d_%H-%M') + '.csv'

        tweet_collections.append(TweetCollection(tweet_start_time, tweet_end_time, tweet_file_path))

        extra_data_start_time = tweet_start_time + _delay_time * 60
        bitly_file_path = _collection_folder + '/bitly_' + datetime.utcfromtimestamp(extra_data_start_time).strftime('%Y-%m-%d_%H-%M') + '.csv'
        retweet_file_path = _collection_folder + '/retweet_' + datetime.utcfromtimestamp(extra_data_start_time).strftime('%Y-%m-%d_%H-%M') + '.csv'

        extra_data_collections.append(ExtraDataCollection(extra_data_start_time, tweet_file_path, bitly_file_path, retweet_file_path))

    return [tweet_collections, extra_data_collections]


def start_tweet_queing(start_time):
    end_time = start_time + _collection_time * 60

    twitter_api = TwitterApi(_log_folder + '/tweets_' + datetime.utcfromtimestamp(start_time).strftime('%Y-%m-%d_%H-%M') + '.log')
    [receive_connection, stream_process] = twitter_api.start_stream(end_time)

    tweet_queuer = TweetQueuer(_log_folder + '/queuer_' + datetime.utcfromtimestamp(start_time).strftime('%Y-%m-%d_%H-%M') + '.log')
    [queue, queuer_process] = tweet_queuer.start_queueing(receive_connection, end_time)

    return [twitter_api, tweet_queuer, stream_process, queuer_process, receive_connection, queue]

if __name__ == "__main__":
    main()
