import sys
import os
from threading import Thread
from datetime import datetime, timedelta
from config_manager import ConfigManager
from logger import Logger
from collection_information import CollectionInformation
from extra_data_retriever import ExtraDataRetriever
from tweet_writer import TweetWriter
from twitter_api import TwitterApi
from tweet_queuer import TweetQueuer


class Master:
    def __init__(self, collection_interval, number_of_collections, delay_time, collection_folder, log_folder):
        self.config_manager = ConfigManager('config.conf')
        self.collection_interval = collection_interval
        self.number_of_collections = number_of_collections
        self.delay_time = delay_time
        self.collection_folder = collection_folder
        self.log_folder = log_folder
        self.logger = Logger(self.log_folder + '/master_' + datetime.utcnow().strftime('%Y-%m-%d_%H-%M') + '.log')

        self.logger.log('Master/init', 'Master created!')

        self.bitly_token_id = 0
        self.queue = None
        self.collection_informations = []
        self.base_time = datetime.utcnow()
        self.stream_process = None
        self.receive_connection = None
        self.queuer_process = None
        self.retrieve_processes = []
        self.collection_end_time = datetime.utcnow() + (timedelta(minutes=self.collection_interval) * self.number_of_collections)
        self.stream_end_time = datetime.utcnow() + (timedelta(minutes=self.collection_interval) * self.number_of_collections) + timedelta(minutes=1) # Add some padding to the stream

        #self.start_up()
        #self.setup_collection_information()

    def start_up(self):
        twitter_api = TwitterApi(self.log_folder + '/tweets_' + self.base_time.strftime('%Y-%m-%d_%H-%M') + '.log')
        [self.receive_connection, self.stream_process] = twitter_api.start_stream(self.stream_end_time)

        tweet_queuer = TweetQueuer(self.log_folder + '/queuer_' + self.base_time.strftime('%Y-%m-%d_%H-%M') + '.log')
        [self.queue, self.queuer_process] = tweet_queuer.start_queueing(self.receive_connection, self.stream_end_time)

    def setup_collection_information(self):
        for i in range(self.number_of_collections):
            start_time = self.base_time + (timedelta(minutes=self.collection_interval) * i)
            extra_data_time = start_time + timedelta(minutes=self.collection_interval) + timedelta(minutes=self.delay_time)
            collection_file_path = self.collection_folder + '/collection_' + start_time.strftime('%Y-%m-%d_%H-%M') + '.csv'
            retweets_file_path = self.collection_folder + '/retweets_' + start_time.strftime('%Y-%m-%d_%H-%M') + '.csv'
            bitly_file_path = self.collection_folder + '/bitly_' + start_time.strftime('%Y-%m-%d_%H-%M') + '.csv'
            ci = CollectionInformation(start_time, extra_data_time, collection_file_path, retweets_file_path, bitly_file_path)

            self.collection_informations.append(ci)

    def run(self):

        self.logger.log('Master/init', 'Master started!')

        collection_thread = Thread(target=self.collect_tweets)
        collect_extra_data_thread = Thread(target=self.collect_extra_data)

        collection_thread.start()
        collect_extra_data_thread.start()

        collection_thread.join()
        collect_extra_data_thread.join()

        print('Will wait for extra data retrieve processes to end!')
        self.logger.log('Master/run', 'Will wait for extra data retrieve processes to end!')
        for retrieve_process in self.retrieve_processes:
            retrieve_process.join()

        print('Will wait for stream process to finish!')
        self.logger.log('Master/run', 'Will wait for stream process to finish!')
        self.stream_process.join()

        self.receive_connection.close()
        self.queuer_process.terminate()

        print('Will wait for queuer process to finish!')
        self.logger.log('Master/run', 'Will wait for queuer process to finish!')
        self.queuer_process.join()

        self.logger.log('Master/init', 'Master done!')

    def collect_tweets(self):
        tweet_writer = TweetWriter()

        while datetime.utcnow() < self.collection_end_time:
            for ci in self.collection_informations:
                if not ci.tweet_collection_started and datetime.utcnow() >= ci.start_time:
                    print('Changing file!')
                    self.logger.log('Master/collect_tweets', 'Changing file!')
                    tweet_writer.change_file(ci.collection_file_path)
                    ci.tweet_collection_started = True

            if not self.queue.empty():
                status = self.queue.get()
                tweet_writer.write(status)

        print('Done collecting tweets')
        self.logger.log('Master/collect_tweets', 'Done collecting tweets')

    def collect_extra_data(self):
        while True:
            all_started = True

            for ci in self.collection_informations:
                if not ci.extra_data_started:
                    all_started = False

                    if datetime.utcnow() >= ci.extra_data_time:
                        extra_data_retriever = ExtraDataRetriever(
                            self.log_folder + '/extra_data_' + ci.start_time.strftime('%Y-%m-%d_%H-%M') + '.log')
                        retrieve_process = extra_data_retriever.start_retrieving(ci.collection_file_path,
                                                                                 ci.retweets_file_path,
                                                                                 ci.bitly_file_path,
                                                                                 self.bitly_token_id)
                        self.retrieve_processes.append(retrieve_process)
                        self.toggle_bitly_token_id()
                        ci.extra_data_started = True

            if all_started:
                break

        print('Done starting extra collections')
        self.logger.log('Master/collect_extra_data', 'Done starting extra collections')

    def toggle_bitly_token_id(self):
        if self.bitly_token_id == 0:
            self.bitly_token_id = 1
        else:
            self.bitly_token_id = 0


def main(collection_interval, number_of_collections, delay_time, collection_folder='data/collection', log_folder='data/logs'):

    print('Setting up folders...')

    os.makedirs(log_folder, exist_ok=True)
    os.makedirs(collection_folder, exist_ok=True)

    print('Master started')

    master = Master(collection_interval, number_of_collections, delay_time, collection_folder, log_folder)
    #master.run()

    print('Master done!')


if __name__ == '__main__':
    if len(sys.argv) == 4:
        main(int(sys.argv[1]), int(sys.argv[2]), int(sys.argv[3]))
    else:
        print('Not the correct amount of arguments provided. Usage: '
              '\'collection_interval (minutes)\' \'number_of_collections\' '
              '\'delay_time (minutes)\'')
