import tweepy
import time
from listener import Listener, StopStreamingException
from logger import Logger
from multiprocessing import Process, Pipe
from config_manager import ConfigManager


_config = ConfigManager('config.conf')
_twitter_config = _config.twitter_configs[0]

class TwitterApi:
    def __init__(self, log_path):
        self.logger = Logger(log_path)

        self.ACCESS_TOKEN = _twitter_config.access_token
        self.ACCESS_SECRET = _twitter_config.access_secret
        self.CONSUMER_KEY = _twitter_config.consumer_key
        self.CONSUMER_SECRET = _twitter_config.consumer_secret

        self.auth = tweepy.OAuthHandler(self.CONSUMER_KEY, self.CONSUMER_SECRET)
        self.auth.set_access_token(self.ACCESS_TOKEN, self.ACCESS_SECRET)

        self.api = None

    def start_stream(self, end_time):
        send_connection, receive_connection = Pipe()

        stream_process = Process(name='Stream Process', target=self.stream, args=(send_connection, end_time))
        stream_process.start()

        return [receive_connection, stream_process]

    def stream(self, connection, end_time):
        while True:
            try:
                stream_listener = Listener(connection, end_time, self.logger)
                stream = tweepy.Stream(auth=self.auth, listener=stream_listener)

                stream.sample()
            except StopStreamingException:
                break
            except Exception as e:
                self.logger.log('Twitter API/stream', 'Exception!', str(e))

        self.logger.log('Twitter API/stream', 'Done streaming')
        exit()

    def create_api(self):
        self.api = tweepy.API(self.auth,
                              wait_on_rate_limit=True,
                              wait_on_rate_limit_notify=True,
                              retry_count=5,
                              retry_delay=60)

    def get_tweets(self, ids):
        tweets = None

        time_to_wait = 1
        successful = False
        while not successful:
            try:
                tweets = self.api.statuses_lookup(ids)
                successful = True
            except Exception as e:
                self.logger.log('twitter_api/get_tweets', 'Connection error to Twitter API, will wait ' + str(time_to_wait) + ' min', e)
                time.sleep(time_to_wait * 60)
                time_to_wait *= 2
                self.create_api()

        return tweets

    def get_retweets(self, ids_to_retrieve):
        retweets = {}

        tweets = self.get_tweets(ids_to_retrieve)
        for tweet in tweets:
            retweets[tweet.id] = tweet.retweet_count

        return retweets
